% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %
% MacKayNeal Function                                                     %
%                                                                         %
% Inputs                                                                  %
% N: length of the codeword (Number of columns of parity-check matrix H)  %
% r: rate of the LDPC code                                                %
% v: distribution of edges                                                %
% h: distribution of nodes                                                %
%                                                                         %
% Outputs                                                                 %
% H: parity-check matrix                                                  %
%                                                                         %
% Author: Aitor Lpez Hernndez                                           %
%                                                                         %
% % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % % %

function H = MacKayNeal(N,r,v,h)

H = zeros(N*(1-r),N); % Creation of an idle parity-check matrix

a = []; % Number of 1s per column of H
for i=1:length(v)
    for j=1:(v(i)*N)
        a = [a,i];
    end
end

b = []; % Number of 1s per row of H
for i=1:length(h)
    for j=1:(h(i)*(N*(1-r)))
        b = [b,i];
    end
end

b_i = 1:length(b); % Indexes of rows of H that still have 1s to append

for i=1:N
    for j=1:1:length(b)
    if b(j) == 0
        b_i = setdiff(b_i,b(j));
    end
    end
    c = datasample(b_i,a(i),'Replace',false);
    c_i = zeros(1,length(b));
    for j=1:a(i) 
        H(c(j),i)=1;
        c_i(1,c(j))=1;
    end
    b = b - c_i;
end

% Removal of length-4 cycles

for i=1:N*(1-r)-1
    for j=i+1:N*(1-r)
       w = and(H(i,:),H(j,:));
       c1 = find(w);
       lc = length(c1);
       if lc > 1
           % If found, flip one 1 to 0 in the row with less number of 1s
           if length(find(H(i,:))) < length(find(H(j,:)))
           % Repeat the process until only one column left
           for cc = 1:(lc-1)
               H(j,c1(cc)) = 0;
           end
           else
           for cc = 1:(lc-1)
               H(i,c1(cc)) = 0;
           end
           end
       end
    end
end
end